function [ cleDrawDatOut ] = cleConvert(cleDrawDatIn, nrPointsOnCircle)
%cleConvert    - Convert Boxes, Circles and Wires to Polygons in cleDrawDat structure.
%
%   cleDrawDat = cleConvert(cleDrawDat, nrPointsOnCircle)
%       nrPointsOnCircle    - Optional number of approximation points on a
%                             full circle.

if (~isstruct(cleDrawDatIn))
    error('CleWin:cleConvert:InputError', ...
          'Incorrect input parameter(s).');
end;

if nargin<2
 nrPointsOnCircle=72;
else
 if (~isnumeric(nrPointsOnCircle))
    error('CleWin:cleConvert:InputError', ...
          'Incorrect input parameter(s).');
 end;
end;

if (~isstruct(cleDrawDatIn))
    error('CleWin:cleConvert:InputError', ...
          'Incorrect input parameter.');
end;

cleDrawDatOut=cleEmpty();

%------------------------------------------------------------
for k=1:length(cleDrawDatIn.cleCircle),
 points= [ (cleDrawDatIn.cleCircle{k}.x+cos(linspace(0,2*pi*...
               (nrPointsOnCircle-1)/nrPointsOnCircle,nrPointsOnCircle))*...
            cleDrawDatIn.cleCircle{k}.radius) ; ...
           (cleDrawDatIn.cleCircle{k}.y+sin(linspace(0,2*pi*...
               (nrPointsOnCircle-1)/nrPointsOnCircle,nrPointsOnCircle))*...
            cleDrawDatIn.cleCircle{k}.radius) ]';
 cleDrawDatOut.clePolygon{end+1}.points=points;
 cleDrawDatOut.clePolygon{end}.layer=cleDrawDatIn.cleCircle{k}.layer;
end;
%------------------------------------------------------------

%------------------------------------------------------------
for k=1:length(cleDrawDatIn.cleBox),
 r=sqrt( (cleDrawDatIn.cleBox{k}.length/2)^2 +...
                                       (cleDrawDatIn.cleBox{k}.width/2)^2);
 phk=angle((cleDrawDatIn.cleBox{k}.width/2)+...
                                      i*(cleDrawDatIn.cleBox{k}.length/2));
 rot= cleDrawDatIn.cleBox{k}.angle;
   
 points= [ (cleDrawDatIn.cleBox{k}.x+sin(rot+phk)*r) ... 
           (cleDrawDatIn.cleBox{k}.y-cos(rot+phk)*r); ...
           (cleDrawDatIn.cleBox{k}.x+sin(rot-phk)*r) ... 
           (cleDrawDatIn.cleBox{k}.y-cos(rot-phk)*r); ...
           (cleDrawDatIn.cleBox{k}.x+sin(rot-pi+phk)*r) ... 
           (cleDrawDatIn.cleBox{k}.y-cos(rot-pi+phk)*r); ...
           (cleDrawDatIn.cleBox{k}.x+sin(rot-pi-phk)*r) ... 
           (cleDrawDatIn.cleBox{k}.y-cos(rot-pi-phk)*r); ];

 cleDrawDatOut.clePolygon{end+1}.points=points;
 cleDrawDatOut.clePolygon{end}.layer=cleDrawDatIn.cleBox{k}.layer;
end;
%------------------------------------------------------------

%------------------------------------------------------------
for k=1:length(cleDrawDatIn.clePolygon),
 cleDrawDatOut.clePolygon{end+1}=cleDrawDatIn.clePolygon{k};
 cleDrawDatOut.clePolygon{end}.layer=cleDrawDatIn.clePolygon{k}.layer;
end;
%------------------------------------------------------------


%------------------------------------------------------------
if length(cleDrawDatIn.cleWire)~=0
,7000
 for k=1:length(cleDrawDatIn.cleWire),
  cleDrawDatA=cleEmpty();
  cleDrawDatB=cleEmpty();

  if (cleDrawDatIn.cleWire{k}.style=='r' | cleDrawDatIn.cleWire{k}.style=='round' ) 
   cleDrawDatA.cleCircle{end+1}.x=cleDrawDatIn.cleWire{k}.points(1,1);
   cleDrawDatA.cleCircle{end}.y=cleDrawDatIn.cleWire{k}.points(1,2);
   cleDrawDatA.cleCircle{end}.radius=cleDrawDatIn.cleWire{k}.width/2;
   cleDrawDatA.cleCircle{end}.layer=cleDrawDatIn.cleWire{k}.layer;
   for j=2:size(cleDrawDatIn.cleWire{k}.points,1)
    cleDrawDatA.cleCircle{end+1}.x=cleDrawDatIn.cleWire{k}.points(j,1);
    cleDrawDatA.cleCircle{end}.y=cleDrawDatIn.cleWire{k}.points(j,2);
    cleDrawDatA.cleCircle{end}.radius=cleDrawDatIn.cleWire{k}.width/2;
    cleDrawDatA.cleCircle{end}.layer=cleDrawDatIn.cleWire{k}.layer;
   
    deltaX=cleDrawDatIn.cleWire{k}.points(j,1)-cleDrawDatIn.cleWire{k}.points(j-1,1);
    deltaY=cleDrawDatIn.cleWire{k}.points(j,2)-cleDrawDatIn.cleWire{k}.points(j-1,2);
    pha=angle(deltaX+i*deltaY);
    lngth=sqrt(deltaX^2+deltaY^2);
   
    cleDrawDatB.cleBox{end+1}.x=cleDrawDatIn.cleWire{k}.points(j-1,1)+deltaX/2;
    cleDrawDatB.cleBox{end}.y=cleDrawDatIn.cleWire{k}.points(j-1,2)+deltaY/2;
    cleDrawDatB.cleBox{end}.length=lngth;  
    cleDrawDatB.cleBox{end}.width=cleDrawDatIn.cleWire{k}.width;
    cleDrawDatB.cleBox{end}.angle=pha;
    cleDrawDatB.cleBox{end}.layer=cleDrawDatIn.cleWire{k}.layer;
   end;
   cleDrawDatOut=cleGroup(cleDrawDatOut,cleMerge(cleConvert(cleDrawDatA,32),cleConvert(cleDrawDatB,32)));
  end;

  if (cleDrawDatIn.cleWire{k}.style=='f' | cleDrawDatIn.cleWire{k}.style=='flat' ) 
   for j=2:(size(cleDrawDatIn.cleWire{k}.points,1)-1)
    deltaX=cleDrawDatIn.cleWire{k}.points(j,1)-cleDrawDatIn.cleWire{k}.points(j-1,1);
    deltaY=cleDrawDatIn.cleWire{k}.points(j,2)-cleDrawDatIn.cleWire{k}.points(j-1,2);
    pha=angle(deltaX+i*deltaY);
    lnght=sqrt(deltaX^2+deltaY^2);
   
    cleDrawDatB.cleBox{end+1}.x=cleDrawDatIn.cleWire{k}.points(j-1,1)+deltaX/2;
    cleDrawDatB.cleBox{end}.y=cleDrawDatIn.cleWire{k}.points(j-1,2)+deltaY/2;
    cleDrawDatB.cleBox{end}.length=lngth;  
    cleDrawDatB.cleBox{end}.width=cleDrawDatIn.cleWire{k}.width;
    cleDrawDatB.cleBox{end}.angle=pha;
    cleDrawDatB.cleBox{end}.layer=cleDrawDatIn.cleWire{k}.layer;

    
    phwa=angle((cleDrawDatIn.cleWire{k}.points(j-1,1)-cleDrawDatIn.cleWire{k}.points(j,1))+i*(cleDrawDatIn.cleWire{k}.points(j-1,2)-cleDrawDatIn.cleWire{k}.points(j,2)));
    phwb=angle((cleDrawDatIn.cleWire{k}.points(j,1)-cleDrawDatIn.cleWire{k}.points(j+1,1))+i*(cleDrawDatIn.cleWire{k}.points(j,2)-cleDrawDatIn.cleWire{k}.points(j+1,2)));
    
    cleDrawDatA=cleGroup(cleDrawDatA,clePolygon([cleDrawDatIn.cleWire{k}.points(j,1)+cleDrawDatIn.cleWire{k}.width/2*sin(phwa)  cleDrawDatIn.cleWire{k}.points(j,2)-cleDrawDatIn.cleWire{k}.width/2*cos(phwa); ...
                                                 cleDrawDatIn.cleWire{k}.points(j,1)+cleDrawDatIn.cleWire{k}.width/2*sin(phwb)  cleDrawDatIn.cleWire{k}.points(j,2)-cleDrawDatIn.cleWire{k}.width/2*cos(phwb); ...    
                                                 cleDrawDatIn.cleWire{k}.points(j,1)-cleDrawDatIn.cleWire{k}.width/2*sin(phwa)  cleDrawDatIn.cleWire{k}.points(j,2)+cleDrawDatIn.cleWire{k}.width/2*cos(phwa); ...
                                                 cleDrawDatIn.cleWire{k}.points(j,1)-cleDrawDatIn.cleWire{k}.width/2*sin(phwb)  cleDrawDatIn.cleWire{k}.points(j,2)+cleDrawDatIn.cleWire{k}.width/2*cos(phwb)])); 
   end;
   deltaX=cleDrawDatIn.cleWire{k}.points(j+1,1)-cleDrawDatIn.cleWire{k}.points(j,1);
   deltaY=cleDrawDatIn.cleWire{k}.points(j+1,2)-cleDrawDatIn.cleWire{k}.points(j,2);
   pha=angle(deltaX+i*deltaY);
   lngth=sqrt(deltaX^2+deltaY^2);
   
   cleDrawDatB.cleBox{end+1}.x=cleDrawDatIn.cleWire{k}.points(j,1)+deltaX/2;
   cleDrawDatB.cleBox{end}.y=cleDrawDatIn.cleWire{k}.points(j,2)+deltaY/2;
   cleDrawDatB.cleBox{end}.length=lngth;  
   cleDrawDatB.cleBox{end}.width=cleDrawDatIn.cleWire{k}.width;
   cleDrawDatB.cleBox{end}.angle=pha;
   cleDrawDatB.cleBox{end}.layer=cleDrawDatIn.cleWire{k}.layer;


   cleDrawDatOut=cleGroup(cleDrawDatOut,cleMerge(cleDrawDatA,cleConvert(cleDrawDatB)));
  end;

  if (cleDrawDatIn.cleWire{k}.style=='e' | cleDrawDatIn.cleWire{k}.style=='extended' ) 
   for j=2:(size(cleDrawDatIn.cleWire{k}.points,1)-1)
    deltaX=cleDrawDatIn.cleWire{k}.points(j,1)-cleDrawDatIn.cleWire{k}.points(j-1,1);
    deltaY=cleDrawDatIn.cleWire{k}.points(j,2)-cleDrawDatIn.cleWire{k}.points(j-1,2);
    pha=angle(deltaX+i*deltaY);
    lngth=sqrt(deltaX^2+deltaY^2);
   
    if (j==2)
     cleDrawDatB.cleBox{end+1}.x=cleDrawDatIn.cleWire{k}.points(j-1,1)+deltaX/2-cleDrawDatIn.cleWire{k}.width/4*cos(pha);
     cleDrawDatB.cleBox{end}.y=cleDrawDatIn.cleWire{k}.points(j-1,2)+deltaY/2-cleDrawDatIn.cleWire{k}.width/4*sin(pha);
     cleDrawDatB.cleBox{end}.lngth=lngth+cleDrawDatIn.cleWire{k}.width/2;  
     cleDrawDatB.cleBox{end}.layer=cleDrawDatIn.cleWire{k}.layer;
    else
     cleDrawDatB.cleBox{end+1}.x=cleDrawDatIn.cleWire{k}.points(j-1,1)+deltaX/2;
     cleDrawDatB.cleBox{end}.y=cleDrawDatIn.cleWire{k}.points(j-1,2)+deltaY/2;
     cleDrawDatB.cleBox{end}.length=lngth;  
     cleDrawDatB.cleBox{end}.layer=cleDrawDatIn.cleWire{k}.layer;
    end;
    cleDrawDatB.cleBox{end}.width=cleDrawDatIn.cleWire{k}.width;
    cleDrawDatB.cleBox{end}.angle=pha;
    cleDrawDatB.cleBox{end}.layer=cleDrawDatIn.cleWire{k}.layer;


    
    phwa=angle((cleDrawDatIn.cleWire{k}.points(j-1,1)-cleDrawDatIn.cleWire{k}.points(j,1))+i*(cleDrawDatIn.cleWire{k}.points(j-1,2)-cleDrawDatIn.cleWire{k}.points(j,2)));
    phwb=angle((cleDrawDatIn.cleWire{k}.points(j,1)-cleDrawDatIn.cleWire{k}.points(j+1,1))+i*(cleDrawDatIn.cleWire{k}.points(j,2)-cleDrawDatIn.cleWire{k}.points(j+1,2)));
    
    cleDrawDatA=cleGroup(cleDrawDatA,clePolygon([cleDrawDatIn.cleWire{k}.points(j,1)+cleDrawDatIn.cleWire{k}.width/2*sin(phwa)  cleDrawDatIn.cleWire{k}.points(j,2)-cleDrawDatIn.cleWire{k}.width/2*cos(phwa); ...
                                                 cleDrawDatIn.cleWire{k}.points(j,1)+cleDrawDatIn.cleWire{k}.width/2*sin(phwb)  cleDrawDatIn.cleWire{k}.points(j,2)-cleDrawDatIn.cleWire{k}.width/2*cos(phwb); ...    
                                                 cleDrawDatIn.cleWire{k}.points(j,1)-cleDrawDatIn.cleWire{k}.width/2*sin(phwa)  cleDrawDatIn.cleWire{k}.points(j,2)+cleDrawDatIn.cleWire{k}.width/2*cos(phwa); ...
                                                 cleDrawDatIn.cleWire{k}.points(j,1)-cleDrawDatIn.cleWire{k}.width/2*sin(phwb)  cleDrawDatIn.cleWire{k}.points(j,2)+cleDrawDatIn.cleWire{k}.width/2*cos(phwb)])); 
   end;
   deltaX=cleDrawDatIn.cleWire{k}.points(j+1,1)-cleDrawDatIn.cleWire{k}.points(j,1);
   deltaY=cleDrawDatIn.cleWire{k}.points(j+1,2)-cleDrawDatIn.cleWire{k}.points(j,2);
   pha=angle(deltaX+i*deltaY);
   lnght=sqrt(deltaX^2+deltaY^2);
   
   cleDrawDatB.cleBox{end+1}.x=cleDrawDatIn.cleWire{k}.points(j,1)+deltaX/2+cleDrawDatIn.cleWire{k}.width/4*cos(pha);
   cleDrawDatB.cleBox{end}.y=cleDrawDatIn.cleWire{k}.points(j,2)+deltaY/2+cleDrawDatIn.cleWire{k}.width/4*sin(pha);
   cleDrawDatB.cleBox{end}.length=lngth+cleDrawDatIn.cleWire{k}.width/2;  
   cleDrawDatB.cleBox{end}.width=cleDrawDatIn.cleWire{k}.width;
   cleDrawDatB.cleBox{end}.angle=pha;
   cleDrawDatB.cleBox{end}.layer=cleDrawDatIn.cleWire{k}.layer;


   cleDrawDatOut=cleGroup(cleDrawDatOut,cleMerge(cleDrawDatA,cleConvert(cleDrawDatB)));
  end;
  
 end;

end;
%------------------------------------------------------------


%------------------------------------------------------------
for k=1:length(cleDrawDatIn.cleRing),
 cleDrawDatOut=cleGroup(cleDrawDatOut,cleMoveLayer(cleSubstract(cleConvert(cleCircle(cleDrawDatIn.cleRing{k}.x,...
                                                                                       cleDrawDatIn.cleRing{k}.y,...
                                                                                       cleDrawDatIn.cleRing{k}.radius+0.5*cleDrawDatIn.cleRing{k}.width),nrPointsOnCircle),...
                                                                  cleConvert(cleCircle(cleDrawDatIn.cleRing{k}.x,...
                                                                                       cleDrawDatIn.cleRing{k}.y,...
                                                                                       cleDrawDatIn.cleRing{k}.radius-0.5*cleDrawDatIn.cleRing{k}.width),nrPointsOnCircle)),cleDrawDatIn.cleRing{k}.layer));
end;
%------------------------------------------------------------


%------------------------------------------------------------
for k=1:length(cleDrawDatIn.cleText),
 cleDrawDatOut.cleText{end+1}=cleDrawDatIn.cleText{k};
 cleDrawDatOut.cleText{end}.layer=cleDrawDatIn.cleText{k}.layer;
end;
%------------------------------------------------------------


%------------------------------------------------------------
for k=1:length(cleDrawDatIn.cleSymbol),
 cleDrawDatOut.cleSymbol{end+1}=cleDrawDatIn.cleSymbol{k};
end;
%------------------------------------------------------------


%------------------------------------------------------------
if nargout < 1,
 cleDraw(cleDrawDatOut)
end;

