function [ cleDrawDat ] = gphDataMatrixPrintf(x,y,width,height,bitsize,format,varargin)
%gphDataMatrixPrintf    - Print formatted text in a DataMatrix.
%
%   [cleDrawDat] = gphDataMatrixPrintf(x, y, width, height, bitsize, format, ...)
%       x           - DataMatrix x position.
%       y           - DataMatrix y position.
%       width       - DataMatrix width in bits
%       height      - DataMatrix height in bits
%       bitsize     - Size of one square bit
%       format      - String containing C language conversion
%                     specifications.
%
%       cleDrawDat  - Optional output structure for further editing.
%                     If not used, data is directly writen to .cle file.

if (~isnumeric(x) | ~isnumeric(y) | ~isnumeric(width) | ~isnumeric(height) | ...
    ~isnumeric(bitsize) | length(x)~=1 | length(y)~=1 | length(width)~=1 | length(height)~=1 |...
    ~isstr(format) )
    error('CleWin:gphDataMatrixPrintf:InputError', ...
          'Incorrect input parameter(s).');
end;

cleDrawDat=cleEmpty();
dataMatrix = gphCoreDataMatrix(sprintf(format,varargin{:}),width,height);

for n=1:size(dataMatrix,1),
 for m=1:size(dataMatrix,2),
  if dataMatrix(n,m)==1,
    cleDrawDat=cleGroup(cleDrawDat,cleBox(x+(n-1)*bitsize,y+(m-1)*bitsize,bitsize,bitsize));
  end;
 end;
end;


%--------------------------------------------------------------------------
if nargout < 1,
 cleDraw(cleDrawDat)
end;