function [ cleDrawDat ] = sctLine(sctDef, points, varargin)
%sctLine    - Draw SCT Line.
%
%   cleDrawDat = sctLine(sctDef, points, roundOfRadius)
%       sctDef           - etc.
%       points           - etc.
%       etc.
%
%       cleDrawDat       - Optional output structure for further editing.
%                          If not used, data is directly writen to CleWin.


% if (~isnumeric(x) | ~isnumeric(y) | ~isnumeric(height) | ~isnumeric(width) | ...
%      length(x)~=1 | length(y)~=1 | length(height)~=1 | length(width)~=1 )
%     error('CleWin:cleBox:InputError', ...
%           'Incorrect input parameter(s).');
% end;

% if nargin>4
%  if (~isnumeric(varargin{1}) |  length(varargin{1})~=1 )
%     error('CleWin:cleBox:InputError', ...
%           'Incorrect input parameter(s).');  
%  end;
% end;

if (size(points,2)~=2)
 points=points';
end;

if (nargin>2)
 roundOfRadius = varargin{1};
else
 roundOfRadius = sctDef.roundOfRadius;
end;

if (sctDef.numberChnls==1) sctDef.seperationChnls=0; end;

cleDrawDat=cleEmpty();
N=size(points,1);

%--- Draw SCT Channels ----------------------------------------------------
anglesStart = sctInternalAngles(points,1);
nodesStart = sctInternalNodes(sctDef,points,1,anglesStart);
[sepsStart,cleRndOf] = sctInternalRoundOf(sctDef,points,1,nodesStart,...
                                                anglesStart,roundOfRadius);
cleGroup(cleDrawDat,cleRndOf);
for n=2:N,
 anglesEnd = sctInternalAngles(points,n);
 nodesEnd = sctInternalNodes(sctDef,points,n,anglesEnd);  
 [sepsEnd,cleRndOf] = sctInternalRoundOf(sctDef,points,n,nodesEnd,...
                                                  anglesEnd,roundOfRadius);

 cleDrawDat = cleGroup(cleDrawDat,cleRndOf,sctInternalLine(sctDef,...
                                   nodesStart,nodesEnd,sepsStart,sepsEnd));

 anglesStart = anglesEnd; nodesStart = nodesEnd; sepsStart = sepsEnd;
end;

%--- Draw Markers ---------------------------------------------------------
if (sctDef.placeMarkers==true)
 cleLayer(sctDef.markerLayer); 
 for n=1:N,
  if (n==1 | n==N)
   cleDrawDat=cleGroup(cleDrawDat,cleSubstract(cleConvert(...
                    cleCircle(points(n,1),points(n,2),sctDef.dashLength+...
            sctDef.dashSeperation),15),cleConvert(cleCircle(points(n,1),...
                    points(n,2),sctDef.dashLength+sctDef.dashSeperation-...
                                               0.5*sctDef.dashWidth),15))); 
   angles=sctInternalAngles(points,n);
   cleDrawDat=cleGroup(cleDrawDat,cleBox(points(n,1),points(n,2),...
     0.5*sctDef.dashWidth,(sctDef.numberChnls-1)*sctDef.seperationChnls+...
                   2*sctDef.dashLength+3*sctDef.dashSeperation,angles(4)));
  else
   cleDrawDat=cleGroup(cleDrawDat,cleCircle(points(n,1),points(n,2),...
                                 sctDef.dashLength+sctDef.dashSeperation));
  end;
 end;
end;

%--------------------------------------------------------------------------
if nargout < 1,
 cleDraw(cleDrawDat);
end;


%% -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
function [ seperations,cleDrawDat ] = sctInternalRoundOf(sctDef,points,...
                                              n,nodes,angles,roundOfRadius)
if (isnan(angles(1)) | isnan(angles(2)))
 seperations = (0.5*sctDef.dashSeperation+0.5*sctDef.dashLength)*...
                                                ones(1,sctDef.numberChnls);
 cleDrawDat = cleEmpty();
else
 cleDrawDat = cleEmpty();
 roundOfException = false;
       
 if (length(roundOfRadius)==1)
  radius=roundOfRadius;
 else
  radius=roundOfRadius(n-1);  
 end;
 
 sigmaCirc=abs(radius/sin(0.5*angles(3)));
 sigmaIsdCrn=sctDef.seperationChnls*(0.5*sctDef.numberChnls-0.5);
 pointCirc(1)=points(n,1)+cos(angles(5))*(sigmaIsdCrn./...
                        sin(abs(0.5*angles(3)))+sigmaCirc);
 pointCirc(2)=points(n,2)+sin(angles(5))*(sigmaIsdCrn./...
                        sin(abs(0.5*angles(3)))+sigmaCirc);

 %-------------------------------------------------                   
 sigmaCheck=sqrt(abs(pointCirc(1)-points(n,1))^2+abs(pointCirc(2)-...
                                                           points(n,2))^2);                  
 seperationCheck = 2.5*sctDef.dashSeperation+2.5*sctDef.dashLength+...
                                        abs(sigmaCheck*cos(0.5*angles(3))); 
 linePreCheck = sqrt(sum(abs(points(n-1,:)-points(n,:)).^2));
 linePostCheck = sqrt(sum(abs(points(n,:)-points(n+1,:)).^2));
 
 if ((seperationCheck>0.5*linePreCheck | seperationCheck>0.5*linePostCheck)&(n~=length(points)-1)&(n~=2))
  roundOfException = true;
 end;
 
 if ((seperationCheck>linePreCheck | seperationCheck>linePostCheck)&((n==length(points)-1)|(n==2)))
  roundOfException = true;
 end;
 
 if (floor((abs(pi-abs(angles(3))-(sctDef.dashSeperation+sctDef.dashLength)/(pi*radius))*radius)/...
         (sctDef.dashSeperation+sctDef.dashLength))<5)
  roundOfException =true;
 end;
 
 %-------------------------------------------------
                        
 if (roundOfException==false),
  for m=1:sctDef.numberChnls,                         
   sigma=sqrt(abs(pointCirc(1)-nodes(m,1))^2+abs(pointCirc(2)-...
                                                            nodes(m,2))^2);
   r=sigma*abs(sin(0.5*angles(3)));

   thetaStart=angles(5)+0.5*pi+0.5*abs(angles(3))+...
                      (sctDef.dashSeperation+sctDef.dashLength)/(0.5*pi*r);
   thetaEnd  =angles(5)+1.5*pi-0.5*abs(angles(3))-...
                      (sctDef.dashSeperation+sctDef.dashLength)/(0.5*pi*r);
   
   nThetaSteps = ceil((abs(thetaEnd-thetaStart)*r)/...
                            (sctDef.dashSeperation+sctDef.dashLength));
   thetaSteps = linspace(thetaStart,thetaEnd,nThetaSteps);
      
   cleLayer(sctDef.dashLayer);
   for s=1:length(thetaSteps),
    cleDrawDat=cleGroup(cleDrawDat,cleBox(pointCirc(1)+...
                 r*cos(thetaSteps(s)),pointCirc(2)+r*sin(thetaSteps(s)),...
               sctDef.dashWidth, sctDef.dashLength, thetaSteps(s)+0.5*pi));
   end;
 
   seperations(m) = 0.5*sctDef.dashSeperation+0.5*sctDef.dashLength+...
                                             abs(sigma*cos(0.5*angles(3)));
  end;
 end;
 
 if (roundOfException==true),
  cleLayer(sctDef.dashLayer);
  cleDrawDat=cleEmpty();
  for m=1:sctDef.numberChnls,
   sigma=-0.5*sctDef.dashLength/tan(0.5*angles(3));
   cleDrawDat=cleGroup(cleDrawDat,cleBox(nodes(m,1)+...
                   sigma*cos(angles(4)),nodes(m,2)+sigma*sin(angles(4)),...
                   sctDef.dashWidth, sctDef.dashLength, angles(4)+0.5*pi));
  end;
 
  seperations = (sctDef.dashSeperation+0.5*sctDef.dashLength+ ...
                          0.5*sctDef.dashWidth*abs(cos(0.5*angles(3)))+ ...
                         0.5*sctDef.dashLength/sin(abs(0.5*angles(3))))*...
                                                ones(1,sctDef.numberChnls);
 end;
 
end;

%% -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
function [ angles ] = sctInternalAngles(points,n)
N=size(points,1);
if (n==1)
 angles(1)=NaN;
 angles(2)=angle( (points(n+1,1)-points(n,1))+...
                                           i*(points(n+1,2)-points(n,2)) );
 angles(3)=pi;
 angles(4)=angles(2)-0.5*pi;
 angles(5)=NaN;
elseif (n==N)
 angles(1)=angle( (points(n-1,1)-points(n,1))+...
                                           i*(points(n-1,2)-points(n,2)) );
 angles(2)=NaN;
 angles(3)=pi;
 angles(4)=angles(1)+0.5*pi;
 angles(5)=NaN;
else
 angles(1)=angle( (points(n-1,1)-points(n,1))+...
                                           i*(points(n-1,2)-points(n,2)) );
 angles(2)=angle( (points(n+1,1)-points(n,1))+...
                                           i*(points(n+1,2)-points(n,2)) );
 angles(3)=angles(1)-angles(2);
 angles(3)=angles(3)-sign(angles(3))*(abs(angles(3))>(pi))*2*pi;
 if (angles(3)<=0)
  angles(4)=angles(2)+0.5*angles(3);
 else
  angles(4)=angles(2)+pi+0.5*angles(3);
  angles(4)=angles(4)-sign(angles(4))*(abs(angles(4))>(pi))*2*pi;
 end;
 angles(5)=0.5*angles(3)+angles(2);
end;

%% -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
function [ nodes ] = sctInternalNodes(sctDef,points,n,angles)
for m=1:sctDef.numberChnls,
 sigma=sctDef.seperationChnls*(m-(sctDef.numberChnls+1)/2);
 nodes(m,1)=points(n,1)+cos(angles(4)).*(sigma./sin(abs(0.5*angles(3))));
 nodes(m,2)=points(n,2)+sin(angles(4)).*(sigma./sin(abs(0.5*angles(3))));
end;

%% -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
function [ cleDrawDat ] = sctInternalLine(sctDef,nodesStart,nodesEnd,...
                                                         sepsStart,sepsEnd)
cleLayer(sctDef.dashLayer);
cleDrawDat=cleEmpty();

for m=1:size(nodesStart,1),
 lineLength = sqrt(sum(abs(nodesStart(m,:)-nodesEnd(m,:)).^2));
 theta = angle( (nodesEnd(m,1)-nodesStart(m,1))+...
                                       i*(nodesEnd(m,2)-nodesStart(m,2)) );

 pathStartLength = sepsStart(m);
 pathEndLength = lineLength-sepsEnd(m);

 pathSteps = linspace(pathStartLength,pathEndLength,...
                                   ceil((pathEndLength-pathStartLength)/...
                              (sctDef.dashSeperation+sctDef.dashLength)) );

 for s=1:length(pathSteps),
  cleDrawDat=cleGroup(cleDrawDat,cleBox(nodesStart(m,1)+...
      pathSteps(s).*cos(theta),nodesStart(m,2)+pathSteps(s).*sin(theta),...
                              sctDef.dashWidth, sctDef.dashLength, theta));
 end;
end;